// Author: Jean-Philippe Beaudet @ S3R3NITY Technology 
//
// Virtuba crud router tests
// CRUD
//
// Version : 0.0.1s
// Copyrigths S3R3NITY Technologies.
//
//========================================================
var chai = require('chai');
var aes256 = require('aes256')
var should = require('chai').should()
var crud = require('../index.js')
var dummy = 1
var config = require("../config.js")
var fs = require('fs')
var TMP_FILE = config.tmp_file
var PASSPHRASE = config.passphrase

describe('#old-version-crud', function() {
	it('add-create;', function() {
		var q = crud.add.create("", crud.toHash("test"))
		var ret = q.actions.create[0]
		ret[0].should.equal("");
		ret[1].should.equal(crud.toHash("test"));
		crud.refresh(function(queue){
		});
	});
	it('add-read;', function() {
		var q = crud.add.read(crud.toHash("test1"))
		var ret = q.actions.read[0]
		ret.should.equal(crud.toHash("test1"));
		crud.refresh(function(queue){
		});
	});
	it('add-update;', function() {
		var q = crud.add.update(crud.toHash("test1"), crud.toHash("test2"))
		var ret = q.actions.update[0]
		ret[0].should.equal(crud.toHash("test1"));
		ret[1].should.equal(crud.toHash("test2"));
		crud.refresh(function(queue){
		});
	});
	it('add-destroy;', function() {
		var q = crud.add.destroy(crud.toHash("test3"))
		var ret = q.actions.destroy[0]
		ret.should.equal(crud.toHash("test3"));
		crud.refresh(function(queue){
		});
	});
	it('refresh;', function() {
		var q = crud.add.read(crud.toHash("test1"))
		var ret = q.actions.read[0]
		ret.should.equal(crud.toHash("test1"))
		q.actions.read.length.should.equal(1)
		var c = crud.refresh(function(queue){
			queue.actions.read.length.should.equal(0);
			})
	});
});

describe('#old-version-send', function() {
	it('compile value && writejson;', function() {
		var key = PASSPHRASE;
		var encrypted = aes256.encrypt(key, '{map:"map"}');
		var q = crud.add.create("", crud.toHash("test"))
		crud.send(crud.toHash("name"), encrypted , function(err){
		})
	});
});
describe('#new-version session', function() {
	it('create and update session;', function() {
		var s= crud.session()
		var user_hash = crud.toHash("username")
		s.addUser(user_hash)
		var q = s.queue(user_hash)
		var c = q.create("", crud.toHash("test"))
		s.saveUser(q)
		var x = s.user()[0].actions.create[0]
		JSON.stringify(x).should.equal(JSON.stringify(["", crud.toHash("test")]));
	})
	it('save session;', function() {
		var s= crud.session()
		//var user_hash = crud.toHash("username")
		var id = s._id
		s.save()
		var file = fs.readFileSync('./var/tmp/'+id+".save",'utf8')
			file.should.equal(JSON.stringify(s));
	})
});
describe('#new-version crud-prototypes', function() {
	it('create;', function() {
		var s= crud.session()
		var user_hash = crud.toHash("username")
		var q = s.queue(user_hash)
		var c = q.create("", crud.toHash("test"))
		var ret = c.q.actions.create[0]
		ret[0].should.equal("");
		ret[1].should.equal(crud.toHash("test"));
		q= q.refresh();
	});
	it('read;', function() {
		var s= crud.session()
		var user_hash = crud.toHash("username")
		var q = s.queue(user_hash)
		var c = q.read( crud.toHash("test"))
		var ret = c.q.actions.read[0]
		ret.should.equal(crud.toHash("test"));
		q= q.refresh();
	});
	it('update;', function() {
		var s= crud.session()
		var user_hash = crud.toHash("username")
		var q = s.queue(user_hash)
		var c = q.update("", crud.toHash("test"))
		var ret = c.q.actions.update[0]
		ret[0].should.equal("");
		ret[1].should.equal(crud.toHash("test"));
		q= q.refresh();
	});
	it('destroy;', function() {
		var s= crud.session()
		var user_hash = crud.toHash("username")
		var q = s.queue(user_hash)
		var c = q.destroy( crud.toHash("test"))
		var ret = c.q.actions.destroy[0]
		ret.should.equal(crud.toHash("test"));
		q= q.refresh();
	});
});
describe('#new-version-send', function() {
	it('send()', function() {
		var s= crud.session()
		var user_hash1 = crud.toHash("newuser")
		var q = s.queue(user_hash1)
		var c = q.create("", crud.toHash("test"))
		var key = PASSPHRASE;
		var encrypted = aes256.encrypt(key, '{map:"map"}');
		q.send(encrypted)
		var file = fs.readFileSync('./var/tmp/'+user_hash1+".json",'utf8')
			file.should.equal(JSON.stringify(c.q));
	});
});
describe('#new-version-multi-queue', function() {
		it('queue multiple times', function() {
		var s= crud.session()
		var bob = crud.toHash("bob")
		var alice = crud.toHash("alice")
		var bob_q = s.queue(bob)
		var alice_q = s.queue(alice)
		var bob_create = bob_q.create("", crud.toHash("bob"))
		var alice_create = alice_q.create("", crud.toHash("alice"))
		var bob_ret = bob_create.q.actions.create[0]
		bob_ret[0].should.equal("");
		bob_ret[1].should.equal(crud.toHash("bob"));
		var alice_ret = alice_create.q.actions.create[0]
		alice_ret[0].should.equal("");
		alice_ret[1].should.equal(crud.toHash("alice"));

		});
});
